local FRAMES_CREATED = 0;

Nameplates = DongleStub("Dongle-Beta0"):New( "Nameplates" );

local L = NPLocals;
local groupnames = {};
local frames = {};

local blockstatus = true;
local status;

function Nameplates:Enable()
	self:RegisterEvent( "RAID_ROSTER_UPDATE", "UpdateUnitMap" );
	self:RegisterEvent( "PARTY_MEMBERS_CHANGED", "UpdateUnitMap" );
		
	self:RegisterEvent( "PLAYER_ENTERING_WORLD" );
	self:RegisterEvent( "PLAYER_LEAVING_WORLD" );
	
	self:RegisterMessage( "NP_BAR_CREATED" );
	self:RegisterMessage( "NP_BAR_MOUSEENTER" );
	self:RegisterMessage( "NP_BAR_MOUSELEAVE" );
	self:RegisterMessage( "NP_BAR_MOBTYPE_CHANGED" );
	self:RegisterMessage( "NP_BAR_TYPECHANGED", "NP_BAR_MOBTYPE_CHANGED" );
	
	self:RegisterMessage( "NP_HEALTH_ONHIDE", "NP_UPDATE_BAR" );
	self:RegisterMessage( "NP_HEALTH_ONSHOW", "NP_UPDATE_BAR" );
	self:RegisterMessage( "NP_HEALTH_COLORCHANGED" );
	self:RegisterMessage( "NP_HEALTH_VALUECHANGED" );

	self:RegisterMessage( "NP_CAST_ONHIDE", "NP_CAST_COMPLETED" );
	self:RegisterMessage( "NP_CAST_VALUECHANGED" );
	self:RegisterMessage( "NP_CAST_COMPLETED" );
	
	self:RegisterMessage( "NP_NAME_TEXTCOLORCHANGED" );
	
	Nameplates:UpdateUnitMap();
	Nameplates:CheckConfiguration();
	
	-- hook for name plate status messages
	if( blockstatus ) then
		hooksecurefunc( "HideNameplates", Nameplates.HostileHidden );
		hooksecurefunc( "HideFriendNameplates", Nameplates.FriendlyHidden );

		hooksecurefunc( "ShowNameplates", Nameplates.HostileShown );
		hooksecurefunc( "ShowFriendNameplates", Nameplates.FriendlyShown );
	end
	
	status = true;
end

function Nameplates:Disable()
	self:UnregisterAllEvents();
	self:UnregisterAllMessages()
	
	self:RegisterMessage( "NP_BAR_CREATED" );
	
	for i=1, FRAMES_CREATED do
		getglobal( "NPText" .. i ):Hide();
	end
	
	status = nil;
end

function Nameplates:Message( msg, color )
	if( not color ) then
		DEFAULT_CHAT_FRAME:AddMessage( msg );
	else
		DEFAULT_CHAT_FRAME:AddMessage( msg, color.r, color.g, color.b );
	end
end

function Nameplates:PLAYER_ENTERING_WORLD()
	blockstatus = nil;
end

function Nameplates:PLAYER_LEAVING_WORLD()
	blockstatus = true;
end

function Nameplates:HostileHidden()
	if( Nameplates_Config.general.showStatus and not blockstatus ) then
		Nameplates:Message( string.format( L.ENEMYPLATES_TOGGLED, L.DISABLED ), ChatTypeInfo["SYSTEM"] );
	end
end

function Nameplates:HostileShown()
	if( Nameplates_Config.general.showStatus and not blockstatus ) then
		Nameplates:Message( string.format( L.ENEMYPLATES_TOGGLED, L.ENABLED ), ChatTypeInfo["SYSTEM"] );
	end
end

function Nameplates:FriendlyShown()
	if( Nameplates_Config.general.showStatus and not blockstatus ) then
		Nameplates:Message( string.format( L.FRIENDLYPLATES_TOGGLED, L.ENABLED ), ChatTypeInfo["SYSTEM"] );
	end
end

function Nameplates:FriendlyHidden()
	if( Nameplates_Config.general.showStatus and not blockstatus ) then
		Nameplates:Message( string.format( L.FRIENDLYPLATES_TOGGLED, L.DISABLED ), ChatTypeInfo["SYSTEM"] );
	end
end

function Nameplates:UpdateUnitMap()
	groupnames = {};

	for i=1, GetNumRaidMembers() do
		groupnames[ UnitName( "raid" .. i ) ] = "raid";
	end
	
	for i=1, GetNumPartyMembers() do
		groupnames[ UnitName( "party" .. i ) ] = "party";
	end
end

function Nameplates:SetBarTexture( bar )
	if( not bar.NPData.textureSet and Nameplates_Config.bar.texture ~= "default" ) then
		bar.NPData.textureSet = true;
		bar:SetStatusBarTexture( "Interface\\AddOns\\Nameplates\\textures\\" .. Nameplates_Config.bar.texture );
	end
end

function Nameplates:SetBarColor( bar )
	if( Nameplates_Config.bar.enabled ) then
		local type = bar.NPData.type;
		if( type == "friendlyPlayer" and groupnames[ bar.NPData.name ]) then
			type = groupnames[ bar.NPData.name ];
		end
	
		if( Nameplates_Config.bar[ type ] ) then
			bar:SetStatusBarColor( Nameplates_Config.bar[ type ].r, Nameplates_Config.bar[ type ].g, Nameplates_Config.bar[ type ].b, Nameplates_Config.bar.opacity );
		end
	end
end

function Nameplates:SetNameTextColor( bar )
	if( not Nameplates_Config.name.enabled ) then
		return;
	end
	
	local nameFrame = select( 5, bar:GetParent():GetRegions() );

	if( bar.NPData.mouseOver ) then
		nameFrame:SetTextColor( Nameplates_Config.name.hoveredColor.r, Nameplates_Config.name.hoveredColor.g, Nameplates_Config.name.hoveredColor.b );

	elseif( groupnames[ bar.NPData.name ] == "party" ) then
		nameFrame:SetTextColor( Nameplates_Config.name.partyColor.r, Nameplates_Config.name.partyColor.g, Nameplates_Config.name.partyColor.b );

	elseif( groupnames[ bar.NPData.name ] == "raid" ) then
		nameFrame:SetTextColor( Nameplates_Config.name.raidColor.r, Nameplates_Config.name.raidColor.g, Nameplates_Config.name.raidColor.b );
	else
		nameFrame:SetTextColor( Nameplates_Config.name.defaultColor.r, Nameplates_Config.name.defaultColor.g, Nameplates_Config.name.defaultColor.b );		
	end
end

function Nameplates:SetBorders( bar )
	if( Nameplates_Config.bar.hideBorder ) then
		select( 1, bar:GetParent():GetRegions() ):Hide();
	end
end

function Nameplates:SetTextBackdrop( bar )
	if( not bar.NPType or Nameplates_Config[ bar.NPType ].bgOpacity == 0.0 ) then
		bar.textFrame:SetBackdropColor( 0, 0, 0, 0 );
		return;
	end
	
	bar.textFrame:SetBackdropColor( Nameplates_Config[ bar.NPType ].bgColor.r, Nameplates_Config[ bar.NPType ].bgColor.g, Nameplates_Config[ bar.NPType ].bgColor.b, Nameplates_Config[ bar.NPType ].bgOpacity );
	bar.textFrame:SetWidth( getglobal( bar.textFrame:GetName() .. "Text" ):GetStringWidth() + 10 );
end

function Nameplates:UpdateBar( bar )
	Nameplates:SetBarTexture( bar );
	Nameplates:SetTextBackdrop( bar );

	if( bar.NPType == "health" ) then
		bar.NPData.name = select( 5, bar:GetParent():GetRegions() ):GetText();
		bar.NPData.level = tonumber( select( 6, bar:GetParent():GetRegions() ):GetText() );

		Nameplates:SetBarColor( bar );
		Nameplates:SetNameTextColor( bar );
		Nameplates:SetBorders( bar );

		Nameplates:NP_HEALTH_VALUECHANGED( nil, bar );
	else
		Nameplates:NP_CAST_VALUECHANGED( nil, bar );
	end
end

function Nameplates:NP_UPDATE_BAR( event, bar )
	Nameplates:UpdateBar( bar );
end

function Nameplates:NP_HEALTH_COLORCHANGED( event, bar )
	Nameplates:SetBarColor( bar );
end

function Nameplates:NP_HEALTH_VALUECHANGED( event, bar )
	if( not Nameplates_Config.health.enabled ) then
		return;
	end
	
	local minValue, maxValue = bar:GetMinMaxValues();
	local currentValue = bar:GetValue();
	
	-- Check for MobHealth3
	if( Nameplates_Config.health.mobHealth and maxValue == 100 and MobHealth3 ) then
		-- Okay, do we have an entry?
		local current, max = MobHealth3:GetUnitHealth( "player", currentValue, maxValue, bar.NPData.name, bar.NPData.level );

		-- Yep!
		if( current > 0 and max > 0 ) then
			currentValue = current;
			maxValue = max;
		end
	end
	
	if( maxValue == 100 ) then
		getglobal( bar.textFrame:GetName() .. "Text" ):SetText( currentValue .. "%" );
		
	elseif( Nameplates_Config.health.type == "diff" ) then
		local diff = maxValue - currentValue;
		if( diff > 0 ) then
			getglobal( "NPText" .. bar.textFrame .. "Text" ):SetText( "-" .. diff );					
		else
			bar.textFrame:Hide();
			return;
		end
	elseif( Nameplates_Config.health.type == "crtmax" ) then
		getglobal( bar.textFrame:GetName() .. "Text" ):SetText( currentValue .. "/" .. maxValue );				

	elseif( Nameplates_Config.health.type == "crt" ) then
		getglobal( bar.textFrame:GetName() .. "Text" ):SetText( currentValue );				
	else
		getglobal( bar.textFrame:GetName() .. "Text" ):SetText( floor( currentValue / maxValue * 100 + 0.5 ) .. "%" );					
	end
	
	bar.textFrame:Show();
end

function Nameplates:NP_BAR_CREATED( event, bar )
	FRAMES_CREATED = FRAMES_CREATED + 1;

	local textFrame = CreateFrame( "Frame", "NPText" .. FRAMES_CREATED, bar, "NPTextTemplate" );
	getglobal( textFrame:GetName() .. "Text"):SetTextColor( Nameplates_Config.health.textColor.r, Nameplates_Config.health.textColor.g, Nameplates_Config.health.textColor.b, Nameplates_Config.health.textOpacity );

	textFrame:SetScale( Nameplates_Config.health.scale );
	textFrame:SetPoint( "TOP", bar, "CENTER", 0, 5 );
	textFrame:Show();
	
	bar.textFrame = textFrame;
	
	Nameplates:UpdateBar( bar );
	
	frames[ bar ] = true;
end

function Nameplates:NP_BAR_MOUSEENTER( event, bar )
	Nameplates:SetNameTextColor( bar );
end

function Nameplates:NP_BAR_MOUSELEAVE( event, bar )
	Nameplates:SetNameTextColor( bar );
end

function Nameplates:NP_BAR_MOBTYPE_CHANGED( event, bar )
	Nameplates:UpdateBar( bar );
end

function Nameplates:NP_CAST_COMPLETED( event, bar )
	bar.textFrame:Hide();
end

function Nameplates:NP_CAST_VALUECHANGED( event, bar )
	if( not Nameplates_Config.cast.enabled ) then
		return;
	end

	local minValue, maxValue = bar:GetMinMaxValues();
	local currentValue = bar:GetValue();

	if( currentValue >= maxValue or currentValue == 0 ) then
		bar.textFrame:Hide();
		return;
	end
	
	maxValue = maxValue - currentValue + ( currentValue - minValue );
	currentValue = math.floor( ( ( currentValue - minValue ) * 100 ) + 0.5 ) / 100;
	
	if( Nameplates_Config.cast.type == "timeleft" ) then
		getglobal( bar.textFrame:GetName() .. "Text" ):SetText( string.format( "%.2f", maxValue - currentValue ) );
		bar.textFrame:Show();
		
	elseif( Nameplates_Config.cast.type == "percent" ) then
		getglobal( bar.textFrame:GetName() .. "Text" ):SetText( string.format( "%d%%", currentValue / maxValue * 100 + 0.05 ) );
		bar.textFrame:Show();
		
	elseif( Nameplates_Config.cast.type == "minmax" ) then
		getglobal( bar.textFrame:GetName() .. "Text" ):SetText( string.format( "%.2f / %.2f", currentValue, maxValue ) );		
		bar.textFrame:Show();		
	else
		bar.textFrame:Hide();
	end
end

-- Check configuration, set slash commands
function Nameplates:CheckConfiguration()
	if( not Nameplates_Config ) then
		Nameplates_Config = {};
	end
	
	if( Nameplates_Config.version ~= L.VERSION ) then
		for category, rows in pairs( Nameplates:GetDefaultConfig() ) do
			if( Nameplates_Config[ category ] == nil ) then
				Nameplates_Config[ category ] = rows;
			elseif( type( rows ) == "table" ) then
				for subKey, subValue in pairs( rows ) do
					if( Nameplates_Config[ category ][ subKey ] == nil ) then
						Nameplates_Config[ category ][ subKey ] = subValue;
					end
				end
			end
		end
	end
	
	Nameplates_Config.version = L.VERSION;
	
	SLASH_NAMEPLATES1 = "/np";
	SLASH_NAMEPLATES2 = "/nameplate";
	SLASH_NAMEPLATES3 = "/nameplates";
	SLASH_NAMEPLATES4 = "/namep";
	SlashCmdList["NAMEPLATES"] = function()
		UIParentLoadAddOn( "SSUI" );
		SSUI:ShowConfig( "np" );
	end
end

function Nameplates_ReloadMod()
	if( not Nameplates_Config.general.enabled and status ) then
		Nameplates:Disable();
		return;
		
	elseif( Nameplates_Config.general.enabled and not status ) then
		Nameplates:Enable();
	end
	
	for bar, _ in pairs( frames ) do
		if( bar.NPData.textureSet and Nameplates_Config.bar.texture == "default" ) then
			bar:SetStatusBarTexture( "Interface\\TargetingFrame\\UI-TargetingFrame-BarFill" );
		end

		bar.NPData.textureSet = nil;

		bar.textFrame:SetScale( Nameplates_Config.health.scale );
		
		getglobal( bar.textFrame:GetName() .. "Text" ):SetTextColor( Nameplates_Config.health.textColor.r, Nameplates_Config.health.textColor.g, Nameplates_Config.health.textColor.b, Nameplates_Config.health.textOpacity );
		select( 1, bar:GetParent():GetRegions() ):Show();
		
		Nameplates:UpdateBar( bar );

		if( not Nameplates_Config[ bar.NPType ].enabled ) then
			bar.textFrame:Hide();
		elseif( bar:IsVisible() ) then
			bar.textFrame:Show();
		end
	end
end

function Nameplates_LoadUI()
	SSUI:RegisterUI( "np", { defaultTab = "NPGeneralConfig", title = L.NAMEPLATES, get = "Nameplates_GetVariable", set = "Nameplates_SetVariable" } );

	SSUI:RegisterTab( "np", "NPGeneralConfig", L.TAB_GENERAL, 1 );
	SSUI:RegisterTab( "np", "NPNameConfig", L.TAB_NAME, 2 );
	SSUI:RegisterTab( "np", "NPTextConfig", L.TAB_HEALTH, 3 );
	SSUI:RegisterTab( "np", "NPCastConfig", L.TAB_CAST, 4 );
	SSUI:RegisterTab( "np", "NPBarConfig", L.TAB_BAR, 5 );
	--SSUI:RegisterTab( "np", "NPHideConfig", L.TAB_HIDE, 6 );
	
	-- Add the elements
	local UIList = {
		-- General configuration
		{ text = L.GENERAL_ENABLE, OnChange = "Nameplates_ReloadMod", type = "check", var = { "general", "enabled" }, parent = "NPGeneralConfig" };
		{ text = L.GENERAL_NPSTATUS, tooltip = L.GENERAL_NPSTATUS_TT, OnChange = "Nameplates_ReloadMod", type = "check", var = { "general", "showStatus" }, parent = "NPGeneralConfig" },
		
		-- Name text
		{ text = L.NAME_ENABLE, OnChange = "Nameplates_ReloadMod", type = "check", var = { "name", "enabled" }, parent = "NPNameConfig" };
		{ text = L.NAME_DEFAULTCOLOR, tooltip = L.NAME_DEFAULTCOLOR_TT, OnChange = "Nameplates_ReloadMod", type = "color", var = { "name", "defaultColor" }, parent = "NPNameConfig" },
		{ text = L.NAME_RAIDCOLOR, tooltip = L.NAME_RAIDCOLOR_TT, OnChange = "Nameplates_ReloadMod", type = "color", var = { "name", "raidColor" }, parent = "NPNameConfig" },
		{ text = L.NAME_PARTYCOLOR, tooltip = L.NAME_PARTYCOLOR_TT, OnChange = "Nameplates_ReloadMod", type = "color", var = { "name", "partyColor" }, parent = "NPNameConfig" },
		{ text = L.NAME_HOVEREDCOLOR, tooltip = L.NAME_HOVEREDCOLOR_TT, OnChange = "Nameplates_ReloadMod", type = "color", var = { "name", "hoveredColor" }, parent = "NPNameConfig" },

		-- Health text
		{ text = L.HEALTH_ENABLE, tooltip = L.HEALTH_ENABLE_TT, OnChange = "Nameplates_ReloadMod", type = "check", var = { "health", "enabled" }, parent = "NPTextConfig" };
		{ text = L.HEALTH_MOBHEALTH, tooltip = L.HEALTH_MOBHEALTH_TT, OnChange = "Nameplates_ReloadMod", type = "check", var = { "health", "mobHealth" }, parent = "NPTextConfig" },
		{ text = L.HEALTH_TYPE, tooltip = L.HEALTH_TYPE_TT, OnChange = "Nameplates_ReloadMod", type = "dropdown",  list = { { "percent", L.PERCENT }, { "diff", L.DIFFERENCE }, { "crtmax", L.CURRENTMAX }, { "crt", L.CURRENT } }, var = { "health", "type" }, parent = "NPTextConfig" },
		{ text = L.TEXT_BACKGROUND, OnChange = "Nameplates_ReloadMod", type = "color", var = { "health", "bgColor" }, parent = "NPTextConfig" },
		{ text = L.TEXT_COLOR,  OnChange = "Nameplates_ReloadMod", type = "color", var = { "health", "textColor" }, parent = "NPTextConfig" },
		{ text = L.TEXT_BGOPACITY, showValue = true, isPercent = true, OnChange = "Nameplates_ReloadMod", type = "slider", var = { "health", "bgOpacity" }, parent = "NPTextConfig" },
		{ text = L.TEXT_OPACITY, showValue = true, isPercent = true, OnChange = "Nameplates_ReloadMod", type = "slider", var = { "health", "textOpacity" }, parent = "NPTextConfig" },
		{ text = L.TEXT_SCALE, showValue = true, isPercent = true, OnChange = "Nameplates_ReloadMod", type = "slider", var = { "health", "scale" }, parent = "NPTextConfig" },
		
		-- Casting bar text
		{ text = L.CAST_ENABLE, tooltip = L.CAST_ENABLE_TT, OnChange = "Nameplates_ReloadMod", type = "check", var = { "cast", "enabled" }, parent = "NPCastConfig" };
		{ text = L.CAST_TYPE, tooltip = L.CAST_TYPE_TT, OnChange = "Nameplates_ReloadMod", type = "dropdown", list = { { "timeleft", L.TIMELEFT }, { "percent", L.PERCENT }, { "minmax", L.CURRENTMAX } }, var = { "cast", "type" }, parent = "NPCastConfig" },
		{ text = L.TEXT_BACKGROUND, OnChange = "Nameplates_ReloadMod", type = "color", var = { "cast", "bgColor" }, parent = "NPCastConfig" },
		{ text = L.TEXT_COLOR,  OnChange = "Nameplates_ReloadMod", type = "color", var = { "cast", "textColor" }, parent = "NPCastConfig" },
		{ text = L.TEXT_BGOPACITY, showValue = true, isPercent = true, OnChange = "Nameplates_ReloadMod", type = "slider", var = { "cast", "bgOpacity" }, parent = "NPCastConfig" },
		{ text = L.TEXT_OPACITY, showValue = true, isPercent = true, OnChange = "Nameplates_ReloadMod", type = "slider", var = { "cast", "textOpacity" }, parent = "NPCastConfig" },
		{ text = L.TEXT_SCALE, showValue = true, isPercent = true, OnChange = "Nameplates_ReloadMod", type = "slider", var = { "cast", "scale" }, parent = "NPCastConfig" },
		
		-- Bar colors
		{ text = L.BAR_ENABLE, OnChange = "Nameplates_ReloadMod", type = "check", var = { "bar", "enabled" }, parent = "NPBarConfig" };
		{ text = L.BAR_HIDEBORDER, tooltip = L.BAR_HIDEBORDER, OnChange = "Nameplates_ReloadMod", type = "check", var = { "bar", "hideBorder" }, parent = "NPBarConfig" },
		{ text = L.BAR_TEXTURE, tooltip = L.BAR_TEXTURE, OnChange = "Nameplates_ReloadMod", type = "dropdown", list = L.TEXTURES,  var = { "bar", "texture" }, parent = "NPBarConfig" },
		{ text = L.BAR_RAID, tooltip = L.BAR_RAID_TT, OnChange = "Nameplates_ReloadMod", type = "color", var = { "bar", "raid" }, parent = "NPBarConfig" },
		{ text = L.BAR_PARTY, tooltip = L.BAR_PARTY_TT, OnChange = "Nameplates_ReloadMod", type = "color", var = { "bar", "party" }, parent = "NPBarConfig" },
		{ text = L.BAR_FRIENDLYPLAYER, tooltip = L.BAR_FRIENDLYPLAYER_TT, OnChange = "Nameplates_ReloadMod", type = "color", var = { "bar", "friendlyPlayer" }, parent = "NPBarConfig" },
		{ text = L.BAR_FRIENDLYNPC, tooltip = L.BAR_FRIENDLYNPC_TT, OnChange = "Nameplates_ReloadMod", type = "color", var = { "bar", "friendlyNPC" }, parent = "NPBarConfig" },
		{ text = L.BAR_HOSTILE, tooltip = L.BAR_HOSTILE_TT, OnChange = "Nameplates_ReloadMod", type = "color", var = { "bar", "hostile" }, parent = "NPBarConfig" },
		{ text = L.BAR_NEUTRALNPC, tooltip = L.BAR_NEUTRALNPC_TT, OnChange = "Nameplates_ReloadMod", type = "color", var = { "bar", "neutralNPC" }, parent = "NPBarConfig" },
		{ text = L.BAR_OPACITY, showValue = true, isPercent = true, minText = SSU_0_PERCENT, maxText = SSU_100_PERCENT, OnChange = "Nameplates_ReloadMod", type = "slider", var = { "bar", "opacity" }, parent = "NPBarConfig" },
	};
	
	-- Add all of the hide types
	--for id, config in pairs( L.HIDE_TYPES ) do
	--	SSUI:RegisterElement( "np", { text = string.format( L.HIDE_OPTION, config[2] ), tooltip = string.format( L.HIDE_OPTION_TT, config[2] ), OnChange = "Nameplates_ReloadMod", type = "check", var = { "hide", config[1] }, parent = "NPHideConfig" } );
	--end
	
	for _, element in pairs( UIList ) do
		SSUI:RegisterElement( "np", element );
	end
end

function Nameplates:GetDefaultConfig( category )
	local Config = {};
	
	Config.general = {};
	Config.general.enabled = true;
	Config.general.showStatus = true;
	
	
	Config.name = {};
	Config.name.enabled = true;
	Config.name.textOpacity = 1.0;
	
	Config.name.defaultColor = { r = 1, g = 1, b = 1 };
	Config.name.hoveredColor = { r = 1, g = 1, b = 0 };
	Config.name.partyColor = { r = 1, g = 1, b = 1 };
	Config.name.raidColor = { r = 1, g = 1, b =  1};
	
	Config.health = {};
	Config.health.enabled = true;
	Config.health.mobHealth = false;

	Config.health.type = "percent";
	Config.health.scale = 0.85;

	Config.health.textColor = { r = 1, g = 1, b = 1 };
	Config.health.textOpacity = 1.0;

	Config.health.bgColor = { r = 0, g = 0, b = 0 };
	Config.health.bgOpacity = 0.75;
	
	Config.cast = {};
	Config.cast.enabled = true;

	Config.cast.type = "minmax";
	Config.cast.scale = 0.85;
	
	Config.cast.bgColor = { r = 0, g = 0, b = 0 };
	Config.cast.bgOpacity = 0.75;

	Config.cast.textColor = { r = 1, g = 1, b = 1 };
	Config.cast.textOpacity = 1.0;
	
	Config.hide = {};
		
	Config.bar = {};
	Config.bar.enabled = true;
	
	Config.bar.opacity = 1.0;
	
	Config.bar.hideBorder = false;
	Config.bar.texture = "default";

	Config.bar.raid = { r = 0, g = 0, b = 1 };
	Config.bar.party = { r = 0, g = 0, b = 1 };

	Config.bar.friendlyPlayer = { r = 0, g = 0, b = 1 };
	Config.bar.friendlyNPC = { r = 0, g = 1, b = 0 };

	Config.bar.hostile = { r = 1, g = 0, b = 0 };
	Config.bar.neutralNPC = { r = 1, g = 1, b =  0 };
	
	if( category ) then
		return Config[ category ];
	end
	
	return Config;
end

function Nameplates_SetVariable( vars, value )
	Nameplates_Config[ vars[1] ][ vars[2] ] = value;
end

function Nameplates_GetVariable( vars )
	if( Nameplates_Config[ vars[1] ] == nil ) then
		Nameplates_Config[ vars[1] ] = Nameplates:GetDefaultConfig( vars[1] );
	end
	
	return Nameplates_Config[ vars[1] ][ vars[2] ];
end