local frames = {};
local mouseover = nil;
local mouseout = nil;
local mousename;

NPLibrary = DongleStub("Dongle-Beta0"):New( "NPLibrary" );

function NPLibrary:Enable()
	if( not self.frame ) then
		self.frame = CreateFrame( "Frame" );
	end
	
	self.frame:SetScript( "OnUpdate", self.OnUpdate );
	
	self:RegisterEvent( "UPDATE_MOUSEOVER_UNIT" );
end

function NPLibrary:Disable()
	self.frame:SetScript( "OnUpdate", nil );
end

function NPLibrary:UPDATE_MOUSEOVER_UNIT()
	mouseover = true;
	mouseout = true;
	mousename = UnitName( "mouseover" );
end

function NPLibrary:SetKnownType( bar )
	if( bar.NPType ~= "health" ) then
		return;
	end
	
	local r, g, b = NPLibrary:GetColor( bar:GetStatusBarColor() );						
	local type = "none";

	if( r == 1 and g == 0 and b == 0 ) then
		type = "hostile";
	elseif( r == 0 and g == 0 and b == 1 ) then
		type = "friendlyPlayer";
	elseif( r == 0 and g == 1 and b == 0 ) then
		type = "friendlyNPC";
	elseif( r == 1 and g == 1 and b == 0 ) then
		type = "neutralNPC";
	end
	
	if( bar.NPData.type ~= type ) then
		bar.NPData.type = type;
		self:TriggerMessage( "NP_BAR_MOBTYPE_CHANGED", bar );
	end
end

-- Debug, don't use
function NPLibrary:GetFrames()
	return frames;
end

local function BarOnShow()
	-- Need to recheck bar type
	if( floor( this:GetValue() ) == this:GetValue() ) then
		if( this.NPType == "cast" ) then
			NPLibrary:TriggerMessage( "NP_BAR_TYPECHANGED", this );
		end

		this.NPType = "health";
	else
		if( this.NPType == "health" ) then
			NPLibrary:TriggerMessage( "NP_BAR_TYPECHANGED", this );
		end

		this.NPType = "cast";
	end
	
	this.NPData.name = select( 5, this:GetParent():GetRegions() ):GetText();

	if( this.NPType == "health" ) then
		NPLibrary:SetKnownType( this );
		NPLibrary:TriggerMessage( "NP_HEALTH_ONSHOW", this );
	
	elseif( this.NPType == "cast" ) then
		NPLibrary:TriggerMessage( "NP_CAST_ONSHOW", this );
	end

	if( this.OnShowFunc ) then
		this.OnShowFunc();
	end
end

local function BarOnHide()
	this.NPData = {};
	
	if( this.NPType == "health" ) then
		NPLibrary:TriggerMessage( "NP_HEALTH_ONHIDE", this );
	elseif( this.NPType == "cast" ) then
		NPLibrary:TriggerMessage( "NP_CAST_ONHIDE", this );
	end

	if( this.OnHideFunc ) then
		this.OnHideFunc();
	end
end

local function HealthValueChanged()
	-- Color changes sometimes when the health bar value does, so we have to check
	local r, g, b = NPLibrary:GetColor( this:GetStatusBarColor() );
	
	if( this.NPData.barColor ) then
		if( r ~= this.NPData.barColor.r or g ~= this.NPData.barColor.g or b ~= this.NPData.barColor.b ) then
			NPLibrary:SetKnownType( this );
			NPLibrary:TriggerMessage( "NP_HEALTH_COLORCHANGED", this );
		end
	end
	
	this.NPData.barColor = { r = r, g = g, b = b };

	NPLibrary:TriggerMessage( "NP_HEALTH_VALUECHANGED", this );
end

local function CastingValueChanged()
	--[[
		Basically, OnHide doesn't offically get fired for around 1-1.5 seconds
		after the cast actually finishes since it fades out of view, however
		we can check to see if current value equals the max value for cast finished
	]]
	
	local value = this:GetValue();
	local min, max = this:GetMinMaxValues();
	
	if( value >= max or ( value == 0 and max == 0 ) ) then
		NPLibrary:TriggerMessage( "NP_CAST_COMPLETED", this );
	end
	
	NPLibrary:TriggerMessage( "NP_CAST_VALUECHANGED", this );
end


local function BarValueChanged()
	if( this.NPType == "health" ) then
		HealthValueChanged();
	elseif( this.NPType == "cast" ) then
		CastingValueChanged();
	end

	if( this.OnValueChangedFunc ) then
		this.OnValueChangedFunc();
	end
end

--[[
	This is a fun little hack, since Blizzard does all
	name plate stuff in C code, we can hook these to get
	information about other mods modifying them
]]

local function StatusBarColorChanged( self, r, g, b, a )
	self.NPData.barColor = { r = r, g = g, b = b, a = a };
	self:SetStatusBarColorFunc( r, g, b, a );
end

local function NameTextColorChanged( self, r, g, b, a )
	self.textColor = { r = r, g = g, b = b, a = a };
	self:SetNameTextColorFunc( r, g, b, a );
end

local function FindHookedFrames( ... )
	for i=1, select( "#", ... ) do
		local bar = select( i, ... );
		if( bar and bar.NPHooked and bar.GetFrameType and bar:GetFrameType() == "StatusBar" and not bar:GetName() and bar:IsVisible() ) then
			return bar;
		end
	end
end

local function FindUnhookedFrames( ... )
	for i=1, select( "#", ... ) do
		local bar = select( i, ... );
		if( bar and not bar.NPHooked and bar.GetFrameType and bar:GetFrameType() == "StatusBar" and not bar:GetName() and bar:IsVisible() ) then
			return bar;
		end
	end
end

local function UpdateFrames( self, ... )
	for i=1, select( "#", ... ) do
		local bar = FindHookedFrames( select( i, ... ):GetChildren() );
		if( bar ) then
			local parent = bar:GetParent();
			local nameFrame = select( 5, parent:GetRegions() );
			
			if( nameFrame and nameFrame.GetText ) then
				local r, g, b = self:GetColor( nameFrame:GetTextColor() );
				
				if( r == 1 and g == 1 and b == 0 ) then
					bar.NPData.opacitySet = nil;
					bar.NPData.mouseOver = true;
					
					self:TriggerMessage( "NP_BAR_MOUSEENTER", bar );
				
				elseif( bar.NPData.mouseOver ) then
					bar.NPData.mouseOver = nil;
					self:TriggerMessage( "NP_BAR_MOUSELEAVE", bar );
				end
			end
		end
	end
end


local function HookFrames( self, ... )
	for i=1, select( "#", ... ) do
		local bar = FindUnhookedFrames( select( i, ... ):GetChildren() );
		if( bar ) then
			local parent = bar:GetParent();
			local nameFrame = select( 5, parent:GetRegions() );

			if( nameFrame and nameFrame.GetText ) then
				-- Find out if it's a health bar or a casting bar
				if( floor( bar:GetValue() ) == bar:GetValue() ) then
					bar.NPType = "health";
				else
					bar.NPType = "cast";
				end
				
				bar.NPData = {};
				bar.OnHideFunc = bar:GetScript( "OnHide" );
				bar.OnShowFunc = bar:GetScript( "OnShow" );
				bar.OnValueChangedFunc = bar:GetScript( "OnValueChanged" );

				bar:SetScript( "OnShow", BarOnShow );
				bar:SetScript( "OnHide", BarOnHide );
				bar:SetScript( "OnValueChanged", BarValueChanged );

				bar.SetStatusBarColorFunc = bar.SetStatusBarColor;
				bar.SetStatusBarColor = StatusBarColorChanged;
				
				frames[ bar ] = true;
				bar.NPHooked = true;

				self:TriggerMessage( "NP_BAR_CREATED", bar );
				
				if( mouseout ) then
					UpdateFrames( self, ... );
				end

				NPLibrary:SetKnownType( bar );
			end
		end
	end
end

local elapsed = 0;
function NPLibrary:OnUpdate()
	elapsed = elapsed + arg1;
	
	if( elapsed > 0.25 ) then
		HookFrames( NPLibrary, WorldFrame:GetChildren() );
		elapsed = 0;
	end
	
	if( mouseover ) then
		UpdateFrames( NPLibrary, WorldFrame:GetChildren() );
		mouseover = nil;
	end
	
	if( mouseout and ( not UnitExists( "mouseover" ) or UnitName( "mouseover" ) ~= mousename ) ) then
		UpdateFrames( NPLibrary, WorldFrame:GetChildren() );
		mouseout = nil;
	end
end

-- GetColor() returns a float like 0.9999978030100465 so we need to get it to round those to 1, and keep everything else the same
function NPLibrary:GetColor( r, g, b, a )
	return floor( r + 0.001 ), floor( g + 0.001 ), floor( b + 0.001 ), floor( a + 0.001 );
end